@@ -186,7 +186,4 @@ def get_seg_counts(gid, outdir, confidence, validate_sg):
 
 def stack_exons(exons1, exons2):
     
-    if len(exons2.shape) > 1:
-        return sp.r_[exons1, exons2]
-    else:
-        return sp.r_[exons1, exons2[sp.newaxis, :]]
+    return sp.r_[exons1, exons2]
