@@ -16,6 +16,7 @@ import numpy.random as npr
 import hashlib
 
 from .alt_splice import quantify
+from .count import get_size_factors
 from .testing import likelihood
 from . import settings
 from .viz import diagnose as plot
@@ -27,108 +28,37 @@ from .helpers import log_progress, decodeUTF8, codeUTF8
 
 TIME0 = time.time()
 
-def get_gene_expression(options, fn_out=None, strain_subset=None):
+def _get_gene_expression(options, fname_exp_hdf5, sample_subset=None):
 
-    if options.verbose:
-        sys.stdout.write('Quantifying gene expression ...\n')
+    IN = h5py.File(fname_exp_hdf5, 'r')
 
-    ### load gene information
-    genes = pickle.load(open(options.fname_genes, 'rb'), encoding='latin1')[0]
-    numgenes = genes.shape[0]
-
-    ### open hdf5 file containing graph count information
-    IN = h5py.File(options.fname_count_in, 'r')
-    strains = IN['strains'][:].astype('str')
-    ### sort by strain ID
-    strain_idx_all = np.argsort(strains)
-    if strain_subset is None:
-        strain_idx = strain_idx_all.copy()
+    if hasattr(options, 'non_alt_norm') and options.non_alt_norm:
+        gene_counts = IN['raw_count_non_alt'][:]
     else:
-        strain_idx = strain_idx_all[np.in1d(strains[strain_idx_all], strain_subset)]
-    gene_counts = np.zeros((numgenes, strain_idx.shape[0]), dtype='float')
-    gene_names = np.array([x.name for x in genes], dtype='str')
-
-    seg_lens = IN['seg_len'][:]
-    gene_ids_segs = IN['gene_ids_segs'][:].astype('int')
-
-    ### no longer assume that the gene_ids_segs are sorted by gene ID
-    s_idx = np.argsort(gene_ids_segs[:, 0], kind='mergesort')
-    _, u_idx = np.unique(gene_ids_segs[s_idx, 0], return_index=True)
-    s_idx = s_idx[u_idx]
-
-    ### iterate over genes
-    for gidx, iidx in enumerate(s_idx):
-
-        if options.verbose:
-            log_progress(gidx, numgenes, 100)
-
-        ### get idx of non alternative segments
-        non_alt_idx = genes[gidx].get_non_alt_seg_ids()
-        seg_idx = np.arange(iidx, iidx + genes[gidx].segmentgraph.seg_edges.shape[0])
-
-        gene_idx = gene_ids_segs[seg_idx, 0]
-        if len(gene_idx.shape) > 0:
-            gene_idx = gene_idx[0]
-
-        assert(decodeUTF8(IN['gene_names'][:][gene_idx]) == genes[gidx].name)
-        assert(genes[gidx].name == gene_names[gidx])
-
-        if options.non_alt_norm:
-            seg_idx = seg_idx[non_alt_idx]
-
-        ### compute gene expression as the read count over all non alternative segments
-        #gene_counts[gidx, :] = np.dot(IN['segments'][seg_idx, :].T, IN['seg_len'][:][seg_idx]) / np.sum(IN['seg_len'][:][seg_idx])
-        if seg_idx.shape[0] > 1:
-            gene_counts[gidx, :] = np.squeeze(np.dot(IN['segments'][seg_idx, :][:, strain_idx].T, seg_lens[seg_idx])) / options.readlen
-        else:
-            gene_counts[gidx, :] = IN['segments'][seg_idx[0], :][strain_idx] * seg_lens[seg_idx] / options.readlen
-        #seg_offset += genes[gidx].segmentgraph.seg_edges.shape[0]
-
+        gene_counts = IN['raw_count'][:]
+    samples = decodeUTF8(IN['samples'][:])
+    gene_ids = decodeUTF8(IN['gene_ids'][:])
+    gene_symbols = decodeUTF8(IN['gene_symbols'][:])
     IN.close()
 
-    if options.verbose:
-        sys.stdout.write('\n... done.\n')
+    if sample_subset is None:
+        sample_idx = np.arange(samples.shape[0])
+    else:
+        sample_idx = np.where(samples == sample_subset[:, np.newaxis])[1]
 
-
-    ### write results to hdf5
-    if fn_out is not None:
-        OUT = h5py.File(fn_out, 'w')
-        OUT.create_dataset(name='all_strains', data=codeUTF8(strains[strain_idx_all]))
-        OUT.create_dataset(name='strains', data=codeUTF8(strains[strain_idx]))
-        OUT.create_dataset(name='genes', data=codeUTF8(gene_names))
-        OUT.create_dataset(name='raw_count', data=gene_counts, compression="gzip")
-        OUT.close()
-
-    return (gene_counts, strains[strain_idx_all], strains[strain_idx], gene_names)
-
-
-def get_size_factors(gene_counts, options):
-
-    if options.verbose:
-        print('Estimating size factors')
-
-    ### take geometric mean of counts
-    gmean = np.exp(np.mean(np.log(gene_counts + 1), axis=1))
-
-    size_factors = []
-    for i in range(gene_counts.shape[1]):
-        idx = gene_counts[:, i] > 0
-        size_factors.append(np.median(gene_counts[idx, i] / gmean[idx]))
-
-    size_factors = np.array(size_factors, dtype='float')
-
-    return size_factors
+    return (gene_counts[:, sample_idx], samples[sample_idx], sample_idx, gene_ids, gene_symbols)
 
 
 def re_quantify_events(options):
-    """This is more a legacy function for testing that requantifies events on a given graph"""
+    """This is a legacy function for testing that requantifies events on a given graph"""
 
-    ev = pickle.load(open(options.fname_events, 'rb'))[0]
+    ev = pickle.load(open(options.fname_event_counts, 'rb'))[0]
     cov = quantify.quantify_from_graph(ev, np.arange(1000), 'exon_skip', options, fn_merge=sys.argv[1])
 
     return cov
 
-def estimate_dispersion_chunk(gene_counts, matrix, sf, options, test_idx, idx, log=False):
+
+def estimate_dispersion_chunk(gene_counts, matrix, sf, options, idx, log=False):
 
     disp_raw = np.empty((idx.shape[0], 1), dtype='float')
     disp_raw.fill(np.nan)
@@ -143,7 +73,7 @@ def estimate_dispersion_chunk(gene_counts, matrix, sf, options, test_idx, idx, l
         disp = 0.1
         resp = gene_counts[i, :].astype('int')
 
-        if sum(resp / sf) < options.min_count or np.mean(resp == 0) > 0.6 or not test_idx[i]:
+        if sum(resp / sf) < options.min_count or np.mean(resp == 0) > 0.6:
             continue
 
         for j in range(10):
@@ -173,7 +103,7 @@ def estimate_dispersion_chunk(gene_counts, matrix, sf, options, test_idx, idx, l
     return (disp_raw, disp_raw_conv, idx)
 
 
-def estimate_dispersion(gene_counts, matrix, sf, options, test_idx, event_type):
+def estimate_dispersion(gene_counts, matrix, sf, options, event_type):
 
     if options.verbose:
         print('Estimating raw dispersions')
@@ -188,7 +118,7 @@ def estimate_dispersion(gene_counts, matrix, sf, options, test_idx, event_type):
         idx_chunks = [np.arange(x, min(x + binsize, gene_counts.shape[0])) for x in range(0, gene_counts.shape[0], binsize)]
 
         try:
-            result = [pool.apply_async(estimate_dispersion_chunk, args=(gene_counts[idx, :], matrix, sf, options, test_idx[idx], idx,)) for idx in idx_chunks]
+            result = [pool.apply_async(estimate_dispersion_chunk, args=(gene_counts[idx, :], matrix, sf, options, idx,)) for idx in idx_chunks]
             res_cnt = 0
             while result:
                 tmp = result.pop(0).get()
@@ -209,7 +139,7 @@ def estimate_dispersion(gene_counts, matrix, sf, options, test_idx, event_type):
             pool.join()
             sys.exit(1)
     else:
-        (disp_raw, disp_raw_conv, _) = estimate_dispersion_chunk(gene_counts, matrix, sf, options, test_idx, np.arange(gene_counts.shape[0]), log=options.verbose)
+        (disp_raw, disp_raw_conv, _) = estimate_dispersion_chunk(gene_counts, matrix, sf, options, np.arange(gene_counts.shape[0]), log=options.verbose)
 
     if np.sum(disp_raw_conv) == 0:
         print('\nERROR: None of the dispersion estimates converged. Exiting.', file=sys.stderr)
@@ -234,7 +164,11 @@ def fit_dispersion(counts, disp_raw, disp_conv, sf, options, dmatrix1, event_typ
     lowerBound = np.percentile(np.unique(disp_raw[index]), 1)
     upperBound = np.percentile(np.unique(disp_raw[index]), 99)
 
+    ### replace NaN with lower bound (will be ignored)
+    disp_raw[np.isnan(disp_raw)] = lowerBound
+
     idx = np.where((disp_raw > lowerBound) & (disp_raw < upperBound))[0]
+    #disp_raw[sp.isnan(disp_raw)] = -1
 
     matrix = np.ones((idx.shape[0], 2), dtype='float')
     matrix[:, 0] /= mean_count[idx].ravel()
@@ -249,8 +183,8 @@ def fit_dispersion(counts, disp_raw, disp_conv, sf, options, dmatrix1, event_typ
     ok_idx = np.where(~np.isnan(disp_fitted))[0]
     disp_fitted[ok_idx] = Lambda[0] / mean_count[ok_idx] + Lambda[1]
 
-    if np.sum(disp_fitted > 0) > 0:
-        print("\nFound dispersion fit")
+    if np.sum(disp_fitted[ok_idx] > 0) > 0:
+        print("Found dispersion fit")
 
     if options.diagnose_plots:
         plot.mean_variance_plot(counts=counts,
@@ -520,10 +454,10 @@ def calculate_varPrior(disp_raw, disp_fitted, idx, varLogDispSamp):
 def run_testing(cov, dmatrix0, dmatrix1, sf, options, event_type, test_idx, r_idx=None):
 
     ### estimate dispersion
-    (disp_raw, disp_raw_conv) = estimate_dispersion(cov, dmatrix1, sf, options, test_idx, event_type)
+    (disp_raw, disp_raw_conv) = estimate_dispersion(cov, dmatrix1, sf, options, event_type)
 
     ### fit dispersion
-    (disp_fitted, Lambda, disp_idx) = fit_dispersion(cov, disp_raw, (disp_raw_conv[:, 0] & test_idx)[:, np.newaxis], sf, options, dmatrix1, event_type)
+    (disp_fitted, Lambda, disp_idx) = fit_dispersion(cov, disp_raw, disp_raw_conv[:, 0][:, np.newaxis], sf, options, dmatrix1, event_type)
 
     ### adjust dispersion estimates
     (disp_adj, disp_adj_conv) = adjust_dispersion(cov, dmatrix1, disp_raw, disp_fitted, disp_idx, sf, options, event_type)
@@ -568,6 +502,7 @@ def run_testing(cov, dmatrix0, dmatrix1, sf, options, event_type, test_idx, r_id
     cov_used = np.array([cov[i, :] if m_idx[i] == 0 else cov[i + offset, :] for i in range(pvals.shape[0])], dtype=cov.dtype)
     disp_raw_used = np.array([disp_raw[i] if m_idx[i] == 0 else disp_raw[i + offset] for i in range(pvals.shape[0])], dtype=disp_raw.dtype)
     disp_adj_used = np.array([disp_adj[i] if m_idx[i] == 0 else disp_adj[i + offset] for i in range(pvals.shape[0])], dtype=disp_adj.dtype)
+    pvals[np.isnan(pvals)] = 1
     pvals[pvals > 1] = 1
 
     return (pvals, cov_used, disp_raw_used, disp_adj_used)
@@ -609,38 +544,18 @@ def spladder_test(options):
 
     options.fname_genes = os.path.join(options.outdir, 'spladder', 'genes_graph_conf%i.%s%s.pickle' % (options.confidence, options.merge, val_tag))
     options.fname_count_in = os.path.join(options.outdir, 'spladder', 'genes_graph_conf%i.%s%s.count.hdf5' % (options.confidence, options.merge, val_tag))
-    options.fname_exp_hdf5 = os.path.join(options.outdir, 'spladder', 'genes_graph_conf%i.%s%s.gene_exp%s.hdf5' % (options.confidence, options.merge, val_tag, non_alt_tag))
+    options.fname_exp_hdf5 = os.path.join(options.outdir, 'spladder', 'genes_graph_conf%i.%s%s.gene_exp.hdf5' % (options.confidence, options.merge, val_tag))
 
-    condition_strains = None
-    if options.subset_samples:
-        condition_strains = np.unique(np.r_[np.array(options.conditionA), np.array(options.conditionB)])
-        _hash = hashlib.sha256()
-        _hash.update(np.unique(condition_strains))
-        options.fname_exp_hdf5 = os.path.join(options.outdir, 'spladder', 'genes_graph_conf%i.%s%s.gene_exp%s.%s.hdf5' % (options.confidence, options.merge, val_tag, non_alt_tag, _hash.hexdigest()))
-    if os.path.exists(options.fname_exp_hdf5):
-        if options.verbose:
-            print('Loading expression counts from %s' % options.fname_exp_hdf5)
-        IN = h5py.File(options.fname_exp_hdf5, 'r')
-        gene_counts = IN['raw_count'][:]
-        gene_strains = decodeUTF8(IN['strains'][:])
-        gene_strains_all = decodeUTF8(IN['all_strains'][:])
-        gene_ids = decodeUTF8(IN['genes'][:])
-        IN.close()
-    else:
-        gene_counts, gene_strains_all, gene_strains, gene_ids = get_gene_expression(options, fn_out=options.fname_exp_hdf5, strain_subset=condition_strains)
+    condition_samples = np.r_[np.array(options.conditionA), np.array(options.conditionB)]
+    if options.verbose:
+        print('Loading expression counts from %s' % options.fname_exp_hdf5)
+        gene_counts, gene_samples, gene_sample_idx, gene_ids, gene_symbols = _get_gene_expression(options, options.fname_exp_hdf5, sample_subset=condition_samples)
 
-    gene_strains = np.array([x.split(':')[1] if ':' in x else x for x in gene_strains])
-    gene_strains_all = np.array([x.split(':')[1] if ':' in x else x for x in gene_strains_all])
+    gene_samples = np.array([x.split(':')[1] if ':' in x else x for x in gene_samples])
 
     ### get index of samples for difftest
-    idx1 = np.where(np.in1d(gene_strains, options.conditionA))[0]
-    idx2 = np.where(np.in1d(gene_strains, options.conditionB))[0]
-    idx1_all = np.where(np.in1d(gene_strains_all, options.conditionA))[0]
-    idx2_all = np.where(np.in1d(gene_strains_all, options.conditionB))[0]
-
-    ### subset expression counts to tested samples
-    gene_counts = gene_counts[:, np.r_[idx1, idx2]]
-    gene_strains = gene_strains[np.r_[idx1, idx2]]
+    idx1 = gene_sample_idx[:len(options.conditionA)]
+    idx2 = gene_sample_idx[len(options.conditionA):]
 
     ### estimate size factors for library size normalization
     sf_ge = get_size_factors(gene_counts, options)
@@ -669,16 +584,22 @@ def spladder_test(options):
         if options.verbose:
             print('Testing %s events' % event_type)
 
-        options.fname_events = os.path.join(options.outdir, 'merge_graphs_%s_C%i.counts.hdf5' % (event_type, options.confidence))
+        options.fname_event_counts = os.path.join(options.outdir, 'merge_graphs_%s_C%i.counts.hdf5' % (event_type, options.confidence))
+        options.fname_events = os.path.join(options.outdir, 'merge_graphs_%s_C%i.pickle' % (event_type, options.confidence))
 
         ### check whether we have any events at all
-        with h5py.File(options.fname_events, 'r') as IN:
+        with h5py.File(options.fname_event_counts, 'r') as IN:
             if not 'conf_idx' in IN:
-                print('SKIPPING: no events of type %s available for testing in file %s\n' % (event_type, options.fname_events), file=sys.stderr)
+                print('SKIPPING: no events of type %s available for testing in file %s\n' % (event_type, options.fname_event_counts), file=sys.stderr)
                 continue
 
         ### quantify events
-        (cov, gene_idx, event_idx, event_ids, event_strains) = quantify.quantify_from_counted_events(options.fname_events, idx1_all, idx2_all, event_type, options, gen_event_ids=False, high_mem=options.high_memory)
+        (cov, psi, gene_idx, event_idx, event_ids, event_samples) = quantify.quantify_from_counted_events(options.fname_event_counts, idx1, idx2, event_type, options, gen_event_ids=False, high_mem=options.high_memory)
+
+        ### load confident events (for coordinate use later)
+        events = None
+        if len(event_idx) > 0:
+            events = pickle.load(open(options.fname_events, 'rb'))[event_idx]
 
         if options.cap_outliers:
             log_counts = np.log2(cov[0] + 1)
@@ -700,8 +621,9 @@ def spladder_test(options):
         sf_ev = get_size_factors(np.vstack(cov), options)
 
         sf = np.r_[sf_ev, sf_ge]
+        #sf = np.r_[sf_ge, sf_ge]
 
-        assert(np.all(gene_strains == event_strains))
+        assert(np.all(gene_samples == event_samples))
 
         ### map gene expression to event order
         curr_gene_counts = gene_counts[gene_idx, :]
@@ -712,6 +634,12 @@ def spladder_test(options):
         k_idx2 = ((np.mean(cov[1][:, idx1.shape[0]:] <= 1, axis=1) <= options.max_0_frac) | \
                   (np.mean(cov[1][:, :idx1.shape[0]] <= 1, axis=1) <= options.max_0_frac))
 
+        ### compute delta PSI
+        naidx = np.all(np.isnan(psi[:, :idx1.shape[0]]), axis=1) | \
+                np.all(np.isnan(psi[:, idx1.shape[0]:]), axis=1)
+        psi[naidx, :] = 0
+        delta_psi = np.nanmean(psi[:, :idx1.shape[0]], axis=1) - np.nanmean(psi[:, idx1.shape[0]:], axis=1)
+    
         k_idx = np.where(k_idx1 | k_idx2)[0]
 
         if options.verbose:
@@ -729,9 +657,10 @@ def spladder_test(options):
             event_ids = [x[k_idx] for x in event_ids]
         k_idx1 = k_idx1[k_idx]
         k_idx2 = k_idx2[k_idx]
+        delta_psi = delta_psi[k_idx]
 
-        cov[0] = np.around(np.hstack([cov[0], curr_gene_counts]))
-        cov[1] = np.around(np.hstack([cov[1], curr_gene_counts]))
+        cov[0] = np.around(np.hstack([cov[0], curr_gene_counts + 0.5]))
+        cov[1] = np.around(np.hstack([cov[1], curr_gene_counts + 0.5]))
         cov = np.vstack(cov)
         if not event_ids is None:
             event_ids = np.hstack(event_ids)
@@ -766,6 +695,15 @@ def spladder_test(options):
                                  figtitle='Count Distributions',
                                  filename=os.path.join(options.plot_dir, 'count_distribution.%s.%s' % (event_type, options.plot_format)),
                                  options=options)
+
+        ### filter for dPSI
+        k_idx3 = (np.absolute(delta_psi) >= options.min_dpsi)
+        test_idx = test_idx & np.hstack((k_idx3, k_idx3))
+        if options.verbose:
+            old = np.sum(k_idx1 | k_idx2)
+            now = np.sum((k_idx1 | k_idx2) & k_idx3)
+            print('Excluding %i fof %i events from testing due to current delta PSI threshold of %.2f. Testing %i events.' % (old - now, old, options.min_dpsi,now))
+
         ### make event splice forms unique to prevent unnecessary tests
         if not event_ids is None:
             event_ids, u_idx, r_idx = np.unique(event_ids, return_index=True, return_inverse=True)
@@ -778,13 +716,15 @@ def spladder_test(options):
         pvals_adj = adj_pval(pvals, options)
 
         ### compute means and fold changes
-        s = event_strains.shape[0]
+        s = event_samples.shape[0]
         m_ev1 = np.nanmean(cov_used[:, :idx1.shape[0]] / sf_ev[:idx1.shape[0]], axis=1)
         m_ev2 = np.nanmean(cov_used[:, idx1.shape[0]:s] / sf_ev[idx1.shape[0]:], axis=1)
-        fc_ev = np.log2(m_ev1) - np.log2(m_ev2)
         m_ge1 = np.nanmean(cov_used[:, s:s+idx1.shape[0]] / sf_ge[:idx1.shape[0]], axis=1)
         m_ge2 = np.nanmean(cov_used[:, s+idx1.shape[0]:] / sf_ge[idx1.shape[0]:], axis=1)
-        fc_ge = np.log2(m_ge1) - np.log2(m_ge2)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            fc_ev = np.log2(m_ev1) - np.log2(m_ev2)
+            fc_ge = np.log2(m_ge1) - np.log2(m_ge2)
         m_all = np.c_[m_ev1, m_ev2, fc_ev, m_ge1, m_ge2, fc_ge]
 
         if options.diagnose_plots:
@@ -809,29 +749,33 @@ def spladder_test(options):
         ###
 
         ### write test summary (what has been tested, which bam files, etc. ...)
-        pickle.dump((gene_strains,
-                      event_strains,
-                      dmatrix0,
-                      dmatrix1,
-                      event_type),
+        pickle.dump({'gene_samples':gene_samples,
+                     'event_samples':event_samples,
+                     'dmatrix0':dmatrix0,
+                     'dmatrix1':dmatrix1,
+                     'event_type':event_type,
+                     'labels':[options.labelA, options.labelB]},
                      open(os.path.join(outdir, 'test_setup_C%i_%s.pickle' % (options.confidence, event_type)), 'wb'), -1)
 
         ### write test results
         s_idx = np.argsort(pvals)
-        header = np.array(['event_id', 'gene', 'p_val', 'p_val_adj', 'mean_event_count_A', 'mean_event_count_B', 'log2FC_event_count', 'mean_gene_exp_A', 'mean_gene_exp_B', 'log2FC_gene_exp'])
-        event_ids = np.array(['%s_%i' % (event_type, i + 1) for i in event_idx], dtype='str')
+        header = np.array(['event_id', 'chrm', 'exon_pos', 'alt_usage', 'gene_id', 'gene_name', 'p_val', 'p_val_adj', 'dPSI', 'mean_event_count_A', 'mean_event_count_B', 'log2FC_event_count', 'mean_gene_exp_A', 'mean_gene_exp_B', 'log2FC_gene_exp'])
+        event_ids = np.array(['%s.%i' % (event_type, i + 1) for i in event_idx], dtype='str')
+
+        event_pos = np.array([_.get_exon_coordinate_strings() for _ in events])
+        event_chrm = np.array([_.chr for _ in events])
 
         out_fname = os.path.join(outdir, 'test_results_C%i_%s.tsv' % (options.confidence, event_type))
         if options.verbose:
             print('Writing test results to %s' % out_fname)
-        data_out = np.c_[event_ids[s_idx], gene_ids[gene_idx[s_idx]], pvals[s_idx].astype('str'), pvals_adj[s_idx].astype('str'), m_all[s_idx, :]]
+        data_out = np.c_[event_ids[s_idx], event_chrm[s_idx], event_pos[s_idx, 0], event_pos[s_idx, 1], gene_ids[gene_idx[s_idx]], gene_symbols[gene_idx[s_idx]], pvals[s_idx].astype('str'), pvals_adj[s_idx].astype('str'), delta_psi[s_idx].astype('str'), m_all[s_idx, :]]
         np.savetxt(out_fname, np.r_[header[np.newaxis, :], data_out], delimiter='\t', fmt='%s')
 
         ### write extended output
         out_fname = os.path.join(outdir, 'test_results_extended_C%i_%s.tsv' % (options.confidence, event_type))
         if options.verbose:
             print('Writing extended test results to %s' % out_fname)
-        header_long = np.r_[header, ['event_count:%s' % x for x in event_strains], ['gene_exp:%s' % x for x in event_strains], ['disp_raw', 'disp_adj']]
+        header_long = np.r_[header, ['event_count:%s' % x for x in event_samples], ['gene_exp:%s' % x for x in event_samples], ['disp_raw', 'disp_adj']]
 
         data_out = np.c_[data_out, (cov_used[s_idx, :] / sf).astype('str'), disp_raw_used[s_idx].astype('str'), disp_adj_used[s_idx].astype('str')]
         np.savetxt(out_fname, np.r_[header_long[np.newaxis, :], data_out], delimiter='\t', fmt='%s')
@@ -850,7 +794,7 @@ def spladder_test(options):
             taken.add(gid)
         ks_idx = np.array(ks_idx)
 
-        data_out = np.c_[event_ids[ks_idx], gene_ids[gene_idx[ks_idx]], pvals[ks_idx].astype('str'), pvals_adj[ks_idx].astype('str'), m_all[ks_idx, :]]
+        data_out = np.c_[event_ids[ks_idx], event_chrm[ks_idx], event_pos[ks_idx, 0], event_pos[ks_idx, 1], gene_ids[gene_idx[ks_idx]], gene_symbols[gene_idx[ks_idx]], pvals[ks_idx].astype('str'), pvals_adj[ks_idx].astype('str'), delta_psi[ks_idx].astype('str'), m_all[ks_idx, :]]
         data_out = np.r_[header[np.newaxis, :], data_out]
         np.savetxt(out_fname, data_out, delimiter='\t', fmt='%s')
 
